@@ -26,9 +26,12 @@ Pre-modification, these were MIT licensed, and are copyright:
     FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
     OTHER DEALINGS IN THE SOFTWARE.
 """
-from rpds import HashTrieSet
+import pickle
+
 import pytest
 
+from rpds import HashTrieSet
+
 HASH_MSG = "Not sure HashTrieSet implements Hash, it has mutable methods"
 
 
@@ -180,3 +183,9 @@ def test_more_set_comparisons():
     assert s <= s
     assert not (s > s)
     assert s >= s
+
+
+def test_pickle():
+    assert pickle.loads(
+        pickle.dumps(HashTrieSet([1, 2, 3, 4]))
+    ) == HashTrieSet([1, 2, 3, 4])
